'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

exports.getVariant = getVariant;
exports.activate = activate;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var IS_BROWSER = typeof window !== 'undefined';

function apiExists() {
    return IS_BROWSER && window['optimizely'];
}

function getActiveExperiments() {
    return apiExists && window['optimizely'].activeExperiments ? window['optimizely'].activeExperiments : [];
}

function getAllExperiments() {
    return apiExists && window['optimizely'].allExperiments ? window['optimizely'].allExperiments : {};
}

function getVariationMap() {
    return apiExists && window['optimizely'].variationMap ? window['optimizely'].variationMap : {};
}

function activateExperiment(EXPERIMENT_ID) {
    try {
        window['optimizely'] = window['optimizely'] || [];
        window['optimizely'].push(["activate", EXPERIMENT_ID]);
    } catch (err) {
        console.error(err);
    }
}

function isEnabled(EXPERIMENT_NAME) {
    var experiment = getExperimentByName(EXPERIMENT_NAME);
    return experiment && experiment.enabled ? true : false;
}

function isDuplicate(EXPERIMENT_NAME) {

    var EXPERIMENTS_WITH_NAME = [];
    var allExperiments = getAllExperiments();
    for (var key in allExperiments) {
        var experiment = allExperiments[key];
        if (experiment.name === EXPERIMENT_NAME) {
            EXPERIMENTS_WITH_NAME.push(key);
        }
    }
    if (EXPERIMENTS_WITH_NAME.length > 1) {
        console.error('Duplicate experiment names occurred, cannot activate exp.');
        return true;
    }
    return false;
}

function getExperimentId(EXPERIMENT_NAME) {
    var EXPERIMENT_ID = null;
    var allExperiments = getAllExperiments();
    for (var id in allExperiments) {
        var experiment = allExperiments[id];
        if (experiment.name === EXPERIMENT_NAME) {
            EXPERIMENT_ID = id;
        }
    }

    return EXPERIMENT_ID;
}

function getExperimentById(EXPERIMENT_ID) {
    var allExperiments = getAllExperiments();
    var experiment = allExperiments[EXPERIMENT_ID];
    return experiment ? experiment : null;
}

function getExperimentByName(EXPERIMENT_NAME) {
    var EXPERIMENT_ID = getExperimentId(EXPERIMENT_NAME);
    return getExperimentById(EXPERIMENT_ID);
}

function isActive(EXPERIMENT_NAME) {
    var EXPERIMENT_ID = getExperimentId(EXPERIMENT_NAME);
    var activeExperiments = getActiveExperiments();
    var found = activeExperiments.filter(function (activeExperimentId) {
        return activeExperimentId === EXPERIMENT_ID;
    });
    return !!found.length;
}

function getVariant(EXPERIMENT_NAME) {
    var variationMap = getVariationMap();
    if (!isDuplicate(EXPERIMENT_NAME) && isEnabled(EXPERIMENT_NAME) && isActive(EXPERIMENT_NAME)) {
        var EXPERIMENT_ID = getExperimentId(EXPERIMENT_NAME);
        return variationMap[EXPERIMENT_ID];
    }
    return null;
}

function activate(EXPERIMENT_NAME) {

    if (IS_BROWSER && isDuplicate(EXPERIMENT_NAME) || !isEnabled(EXPERIMENT_NAME)) {
        return false;
    }

    var EXPERIMENT_ID = getExperimentId(EXPERIMENT_NAME);

    if (!isActive(EXPERIMENT_NAME) && EXPERIMENT_ID) {
        activateExperiment(EXPERIMENT_ID);
    }

    return isActive(EXPERIMENT_NAME);
}

exports.default = function (experimentName) {
    var suppressActivateOnRender = arguments.length <= 1 || arguments[1] === undefined ? false : arguments[1];
    return function (Wrapped) {
        var Wrapper = function (_Component) {
            (0, _inherits3.default)(Wrapper, _Component);

            function Wrapper() {
                (0, _classCallCheck3.default)(this, Wrapper);
                return (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(Wrapper).apply(this, arguments));
            }

            (0, _createClass3.default)(Wrapper, [{
                key: 'render',
                value: function render() {
                    var isActive = false;
                    if (!suppressActivateOnRender) {
                        isActive = activate(experimentName);
                    }

                    var variant = getVariant(experimentName);

                    return _react2.default.createElement(Wrapped, (0, _extends3.default)({ isActive: isActive,
                        variant: variant }, this.props));
                }
            }]);
            return Wrapper;
        }(_react.Component);

        Wrapper.propTypes = {
            experimentName: _react.PropTypes.string,
            suppressActivateOnRender: _react.PropTypes.bool
        };

        return Wrapper;
    };
};